;; Pipeline model for LoongArch gs464v cores.

;; Copyright (C) 2021 Free Software Foundation, Inc.
;;
;; This file is part of GCC.
;;
;; GCC is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published
;; by the Free Software Foundation; either version 3, or (at your
;; option) any later version.
;;
;; GCC is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
;; License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GCC; see the file COPYING3.  If not see
;; <http://www.gnu.org/licenses/>.

;; Uncomment the following line to output automata for debugging.
;; (automata_option "v")

;; Automaton for integer instructions.
(define_automaton "gs464v_a_alu")

;; Automaton for floating-point instructions.
(define_automaton "gs464v_a_falu")

;; Automaton for memory operations.
(define_automaton "gs464v_a_mem")

;; Describe the resources.

(define_cpu_unit "gs464v_alu1" "gs464v_a_alu")
(define_cpu_unit "gs464v_alu2" "gs464v_a_alu")
(define_cpu_unit "gs464v_mem1" "gs464v_a_mem")
(define_cpu_unit "gs464v_mem2" "gs464v_a_mem")
(define_cpu_unit "gs464v_falu1" "gs464v_a_falu")
(define_cpu_unit "gs464v_falu2" "gs464v_a_falu")

;; Describe instruction reservations.

(define_insn_reservation "gs464v_arith" 1
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "arith,clz,const,logical,
			move,nop,shift,signext,slt"))
  "gs464v_alu1 | gs464v_alu2")

(define_insn_reservation "gs464v_branch" 1
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "branch,jump,call,condmove,trap"))
  "gs464v_alu1 | gs464v_alu2")

(define_insn_reservation "gs464v_imul" 7
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "imul"))
  "gs464v_alu1 | gs464v_alu2")

(define_insn_reservation "gs464v_idiv_si" 12
  (and (match_test "TARGET_ARCH_GS464V")
       (and (eq_attr "type" "idiv")
	    (eq_attr "mode" "SI")))
  "gs464v_alu1 | gs464v_alu2")

(define_insn_reservation "gs464v_idiv_di" 25
  (and (match_test "TARGET_ARCH_GS464V")
       (and (eq_attr "type" "idiv")
	    (eq_attr "mode" "DI")))
  "gs464v_alu1 | gs464v_alu2")

(define_insn_reservation "gs464v_load" 4
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "load"))
  "gs464v_mem1 | gs464v_mem2")

(define_insn_reservation "gs464v_gpr_fp" 16
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "mftg,mgtf"))
  "gs464v_mem1")

(define_insn_reservation "gs464v_fpload" 4
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "fpload"))
  "gs464v_mem1 | gs464v_mem2")

(define_insn_reservation "gs464v_prefetch" 0
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "prefetch,prefetchx"))
  "gs464v_mem1 | gs464v_mem2")

(define_insn_reservation "gs464v_store" 0
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "store,fpstore,fpidxstore"))
  "gs464v_mem1 | gs464v_mem2")

(define_insn_reservation "gs464v_fadd" 4
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "fadd,fmul,fmadd"))
  "gs464v_falu1 | gs464v_falu2")

(define_insn_reservation "gs464v_fcmp" 2
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "fabs,fcmp,fmove,fneg"))
  "gs464v_falu1 | gs464v_falu2")

(define_insn_reservation "gs464v_fcvt" 4
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "fcvt"))
  "gs464v_falu1 | gs464v_falu2")

(define_insn_reservation "gs464v_fdiv_sf" 12
  (and (match_test "TARGET_ARCH_GS464V")
       (and (eq_attr "type" "fdiv,frdiv,fsqrt,frsqrt")
	    (eq_attr "mode" "SF")))
  "gs464v_falu1 | gs464v_falu2")

(define_insn_reservation "gs464v_fdiv_df" 19
  (and (match_test "TARGET_ARCH_GS464V")
       (and (eq_attr "type" "fdiv,frdiv,fsqrt,frsqrt")
	    (eq_attr "mode" "DF")))
  "gs464v_falu1 | gs464v_falu2")

;; Force single-dispatch for unknown or multi.
(define_insn_reservation "gs464v_unknown" 1
  (and (match_test "TARGET_ARCH_GS464V")
       (eq_attr "type" "unknown,multi,atomic,syncloop"))
  "gs464v_alu1 + gs464v_alu2 + gs464v_falu1
   + gs464v_falu2 + gs464v_mem1 + gs464v_mem2")

;; End of DFA-based pipeline description for gs464v
